;+
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2007
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;
; read_l3_hdf
;
; PURPOSE:
;	Reads a TES L3 HDF  data product file and
;	returns the data fields (version F01).  Data are returned in simple
;	structures of simple arrays with no processing, filtering,
;	etc.
;
;	IDL 6.3+ REQUIRED to read version F01 files.
;
; CALL:
; 	read_l3_hdf,file, $
;                nadirgrid=nadirgrid, $
;                nadirdaygrid=nadirdaygrid, $
;                nadirnightgrid=nadirnightgrid, $
;                limbgrid=limbgrid, $
;                limbdaygrid=limbdaygrid, $
;                limbnightgrid=limbnightgrid, $
;                file_attributes = file_attributes, $
;                hdfeos_data = hdfeos_data, $
;                status = status,read_ph=read_ph, $
;                show_options = show_options, $
;                list_grids = list_grids, $
;                actual_grids = actual_grids
;
;
; INPUTS:
;	file = name of file to be read (string)
;
; KEYWORD INPUT:
;
;	read_ph = if set, read production history.  If not set, the
;		production history will be a null string
;
;	show_options : if this keyword is set, a list of the available
;		swaths will be printed to the screen.  All other input
;		will be ignored in this case.
;
;	list_grids : if this keyword is set, a list of the grids
;		present in the file will be printed to the command line.
;
;
; OUTPUT:
;	(none)
;
; KEYWORD OUTPUT:
;	actual_grids : a string array of the names of the grids
;		found in the input file
;
;	The complete list of possible outputs is shown below.
;	Note that different file types (daily, 8-day, monthly)
;	and different species have different subsets of this master
;	list.  The user should read carefully for each application.
;	The types/spieces where each field is available is listed
;	in parenthesis for each entry.  (all) signifies this field is
;	found in all species/type combination
;
;		species  (species data, e.g. O3, CO, etc) (all)
;		AtSurface (species data at surface) (daily, 8 day)
;		SurfaceTemperature (daily ATM-TEMP)
;		SuracePressure (daily)
;		Latitude (all)
;		Longitude (all)
;		Pressure (all)
;		OzoneTropColumn (daily, monthly O3)
;		TotalColumnDensity (all except ATM-TEMP (any type) files)
;		DataCount (8-day, monthly) (as <species>DataCount in file)
;		StdDeviation (8-day, monthly) (as <species>StdDeviation in file)
;		Maximum (8-day, monthly) (as <species>Maximum in file)
;		Minimum (8-day, monthly) (as <species>Minimum in file)
;		TotColDensDataCount (8-day, monthly except ATM-TEMP)
;		TotColDensStdDeviation (8-day, monthly except ATM-TEMP)
;		TotColDensMaximum (8-day, monthly except ATM-TEMP)
;		TotColDensMinimum (8-day, monthly except ATM-TEMP)
;		Projection (all)
;		GridOrigin (all)
;		GridSpacing (all)
;		GridSpacingUnit (all)
;		GridSpan (all)
;		GridSpanUnit (all)
;		TriangleVertDistMax (daily)
;		GlobalSurveyL3Algorithm (daily)
;		DeltaFullLatitude (8-day, monthly)
;		DeltaFullLongitude (8-day, monthly)
;		L3Algorithm (8-day) (as 8-DayL3Algorithm in file)
;		MonthlyL3Algorithm (monthly)
;
;
;    Note:  All anc_data fields, species_data fields, and geo_data
;    	fields have the following attributes in the HDF file.  This
;    	program does NOT return them.
;
;		_FillValue
;		MissingValue
;		Title
;		Units
;	 	UniqueFieldDefinition
;
;	The code hooks are in place, but no attemp to keep track of
;	them has been made.  The program author does not anticipate
;	user demand for these and decided to skip the complexity of
;	having them in the returned data.  Since they are all read,
;	a user wanting this information needs only to collect and
;	return the information (by creating a altered version of this
;	program).
;
;
;	file_attributes = a structure containing the information
;		contained in the file attributes group in the
;		additional group.  The fields in this structure are:
;
;                  ProductionHistory  (not read by default--see input
;                      keywords--this is added by the TES SIPS and is thus
;                      not present in files produced by the TES SCF)
;                  InstrumentName
;                  ProcessLevel
;		   OrbitNumber
;		   OrbitPeriod
;                  TAI93At0zOfGranule (TAI93 At "zero" z Of Granule)
;                  GlobalSurveyNumber
;                  GranuleMonth
;                  GranuleDay
;                  GranuleYear
;                  SurveyMode
;                  PGEVersion
;                  StartUTC
;                  EndUTC
;                  Period
;		   Command_Seq_ID
;
;	hdfeos_data = a structure that contains the information found
;		in the HDFEOS Information group.  The fields are:
;              		StructMetadata
;			coremetadata
;			HDFEOSVersion
;
;	status = Contains success or failure status of the read
;		 0 for success
;		 1 if file does not exist/file not found
;		 2 the file name does not have  F01 in name
;		 3 the file is not HDF 5
;		 4 found 'B.h5' in file, indicating Browse file (not
;		     supported by this reader)
;		 5 the file name does not have the expected .he5
;		     extension
;		 6 File is not L3 type file (L3 not found in file name)
;		 7 IDL version incompatability
;
; NOTE:  On any error except those listed, the program will stop in
; the standard IDL fashion at the error location (does not return to
; the caller).  No other error handling is built into this procedure other
; than those shown in the  status list.
;
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 06/2007
;
; UPDATE HISTORY:
;
;	Brendan Fisher August, 2007, Initial version
;
;-

pro read_l3_hdf,file, $
                nadirgrid=nadirgrid, $
                nadirdaygrid=nadirdaygrid, $
                nadirnightgrid=nadirnightgrid, $
                limbgrid=limbgrid, $
                limbdaygrid=limbdaygrid, $
                limbnightgrid=limbnightgrid, $
                file_attributes = file_attributes, $
                hdfeos_data = hdfeos_data, $
                status = status,read_ph=read_ph, $
                show_options = show_options, $
                list_grids = list_grids, $
                actual_grids = actual_grids

if keyword_set(show_options) then begin
    print,'Available grid names: '
    print,'NadirGrid'
    print,'NadirDayGrid'
    print,'NadirNightGrid'
    print,'LimbGrid'
    print,'LimbDayGrid'
    print,'LimbNightGrid'

    return
endif


status=0
if n_elements(read_ph) eq 0 then read_ph = 0

; check for file existence
ff = file_search(file,count=count)
if count eq 0 then begin
    print,'Cannot find file:'
    print,file
    status = 1
    return
endif

; check version
pos1 = strpos(file,'_F01_')
if pos1 eq -1 then begin
    print,'Expected version _F01_, in file name not found.'
    print,file
    print,'This file format not supported.'
    status = 2
    return
endif
if pos1 ne -1 then version = 1

; check to see if file is an HDF5
if not h5f_is_hdf5(file) then begin
    print,'File is not HDF 5:'
    print,file
    status=3
    return
endif

; check type
spos = strpos(file,'L3')
if spos eq -1 then begin
    print,'File does not seem to be a L3-type product file.'
    print,'File name: ',file
    status=6
    return
end
; make sure it is not a browse L3 file
bpos = strpos(file,'B.h5')
if bpos ne -1 then begin
    print,'File has B.h5 extention, is a Browse product.'
    print,'Please use browse product file reader.'
    print,'File name: ',file
    status = 4
    return
endif
; check for .he5 extension
epos = strpos(file,'.he5')
if epos eq -1 then begin
    print,'Expected .he5 extension not found.'
    print,'File name: ',file
    status = 5
    return
endif

; IDL version checking
iversion = !version
idl_version = float(iversion.release)
if version ge 1 and idl_version lt 6.3 then begin
    print,'IDL v6.3+ required for TES L3 format version F01 files.'
    status = 7
    return
endif
;
; parse filename to get species for species dependent reading
p1 = strpos(file,'TES-Aura_L3')
p2 = strpos(file,'-',p1+11)
p3 = strpos(file,'_',p2+1)
p4 = strpos(file,'-',p2+1)
; if a dash is found before an underscore,
; the file is a montly or ATM-TEMP
if p4 lt p3 and p4 ne -1 then $
  species_name = strmid(file,p2+1,(p4-1)-(p2+1)+1) $
else $
  species_name = strmid(file,p2+1,(p3-1)-(p2+1)+1)
if species_name eq 'ATM' then species_name = 'TATM'
; print,species_name
;
; determine type from file name (Daily, 8D, or Monthly)
;
type = 'daily' ; default
pos = strpos(file,'8D')
if pos ne -1 then type = '8d'
pos = strpos(file,'-M')
if pos ne -1 then type = 'monthly'
;
; open file to data fields
;
fid = h5f_open(file)
hdfid = h5g_open(fid,'HDFEOS')
gridid = h5g_open(hdfid,'GRIDS')

; check which swaths are in file
ngroup = h5g_get_nmembers(fid,'/HDFEOS/GRIDS/')
names = strarr(ngroup)
for i=0,ngroup-1 do names[i] = h5g_get_member_name(fid,'/HDFEOS/GRIDS/',i)
actual_grids = names
if keyword_set(list_grids) then begin
    print,'Grids present in file:'
    for i=0,ngroup-1 do print,names[i]
    h5g_close,gridid
    h5g_close,hdfid
    h5f_close,fid

    return
endif
;print,names
;
; possible grid names
;
; NadirGrid
; NadirDayGrid
; NadirNightGrid
; LimbGrid
; LimbDayGrid
; LimbNightGrid
;
all_grids = ['NadirGrid', $
             'NadirDayGrid', $
             'NadirNightGrid', $
             'LimbGrid', $
             'LimbDayGrid', $
             'LimbNightGrid']

;
; read the data
;
; try each grid type
;
grid = all_grids[0] ; NadirGrid

; this section repeats for all grid types except the
; names of output structures change to match the grid name
nspos = where(names eq grid)
if nspos ne -1 then begin
    case type of
        'daily' : begin         ; separate by type
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'SurfacePressure')
            SurfacePressure = h5d_read(dataid)
            get_data_attributes,dataid,size(SurfacePressure,/type),attrib7
            h5d_close,dataid

            if species_name eq 'TATM' then begin
                dataid = h5d_open(fieldid,'SurfaceTemperature')
                SurfaceTemperature = h5d_read(dataid)
                get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib8
                h5d_close,dataid
            endif else begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid
            endelse

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'TriangleVertDistMax')
            TriangleVertDistMax = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GlobalSurveyL3Algorithm')
            GlobalSurveyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid
            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    nadirgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                SurfaceTemperature : temporary(SurfaceTemperature), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }


                end
                'O3' : begin
                    nadirgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }
                end
                else : begin
                    nadirgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }

                end
            endcase             ; daily species case for output
        end                     ; end daily file case for NadirGrid
        'monthly' : begin       ; monthly file case for NadirGrid
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            if species_name ne 'TATM' then begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensDataCount')
                TotColDensDataCount = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensStdDeviation')
                TotColDensStdDeviation = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMaximum')
                TotColDensMaximum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMinimum')
                TotColDensMinimum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
                h5d_close,dataid
            endif

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'MonthlyL3Algorithm')
            MonthlyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    nadirgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }


                end
                'O3' : begin
                    nadirgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
                else : begin
                    nadirgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
            endcase             ; montly species case for output

        end
        '8d' : begin
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotalColumnDensity')
            TotalColumnDensity = h5d_read(dataid)
            get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensDataCount')
            TotColDensDataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensStdDeviation')
            TotColDensStdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMaximum')
            TotColDensMaximum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMinimum')
            TotColDensMinimum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
            h5d_close,dataid

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'8-DayL3Algorithm')
            L3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            nadirgrid = { $
                        species : temporary(species), $
                        AtSurface : temporary(AtSurface), $
                        DataCount : temporary(DataCount), $
                        StdDeviation : temporary(StdDeviation), $
                        Maximum : temporary(Maximum), $
                        Minimum : temporary(Minimum), $
                        Latitude : temporary(Latitude), $
                        Longitude : temporary(Longitude), $
                        Pressure : temporary(Pressure), $
                        TotalColumnDensity : temporary(TotalColumnDensity), $
                        TotColDensDataCount : temporary(TotColDensDataCount), $
                        TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                        TotColDensMaximum : temporary(TotColDensMaximum), $
                        TotColDensMinimum : temporary(TotColDensMinimum), $
                        Projection : temporary(Projection), $ ; grid attributes
                        GridOrigin : temporary(GridOrigin), $
                        GridSpacing : temporary(GridSpacing), $
                        GridSpacingUnit : temporary(GridSpacingUnit), $
                        GridSpan : temporary(GridSpan), $
                        GridSpanUnit : temporary(GridSpanUnit), $
                        DeltaFullLatitude : temporary(DeltaFullLatitude), $
                        DeltaFullLongitude : temporary(DeltaFullLongitude), $
                        L3Algorithm : temporary(L3Algorithm) $
                        }



        end                     ; 8-day type
    endcase ; NadirGrid file type case
endif ; end if NadirGrid found

; end of repeat section

;---------------
grid = all_grids[1] ; NadirDayGrid

; this section repeats for all grid types except the
; names of output structures change to match the grid name
nspos = where(names eq grid)
if nspos ne -1 then begin
    case type of
        'daily' : begin         ; separate by type
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'SurfacePressure')
            SurfacePressure = h5d_read(dataid)
            get_data_attributes,dataid,size(SurfacePressure,/type),attrib7
            h5d_close,dataid

            if species_name eq 'TATM' then begin
                dataid = h5d_open(fieldid,'SurfaceTemperature')
                SurfaceTemperature = h5d_read(dataid)
                get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib8
                h5d_close,dataid
            endif else begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid
            endelse

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'TriangleVertDistMax')
            TriangleVertDistMax = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GlobalSurveyL3Algorithm')
            GlobalSurveyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid
            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    nadirdaygrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                SurfaceTemperature : temporary(SurfaceTemperature), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }


                end
                'O3' : begin
                    nadirdaygrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }
                end
                else : begin
                    nadirdaygrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }

                end
            endcase             ; daily species case for output
        end                     ; end daily file case for NadirGrid
        'monthly' : begin       ; monthly file case for NadirGrid
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            if species_name ne 'TATM' then begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensDataCount')
                TotColDensDataCount = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensStdDeviation')
                TotColDensStdDeviation = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMaximum')
                TotColDensMaximum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMinimum')
                TotColDensMinimum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
                h5d_close,dataid
            endif

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'MonthlyL3Algorithm')
            MonthlyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    nadirdaygrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }


                end
                'O3' : begin
                    nadirdaygrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
                else : begin
                    nadirdaygrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
            endcase             ; montly species case for output

        end
        '8d' : begin
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotalColumnDensity')
            TotalColumnDensity = h5d_read(dataid)
            get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensDataCount')
            TotColDensDataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensStdDeviation')
            TotColDensStdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMaximum')
            TotColDensMaximum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMinimum')
            TotColDensMinimum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
            h5d_close,dataid

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'8-DayL3Algorithm')
            L3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            nadirdaygrid = { $
                        species : temporary(species), $
                        AtSurface : temporary(AtSurface), $
                        DataCount : temporary(DataCount), $
                        StdDeviation : temporary(StdDeviation), $
                        Maximum : temporary(Maximum), $
                        Minimum : temporary(Minimum), $
                        Latitude : temporary(Latitude), $
                        Longitude : temporary(Longitude), $
                        Pressure : temporary(Pressure), $
                        TotalColumnDensity : temporary(TotalColumnDensity), $
                        TotColDensDataCount : temporary(TotColDensDataCount), $
                        TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                        TotColDensMaximum : temporary(TotColDensMaximum), $
                        TotColDensMinimum : temporary(TotColDensMinimum), $
                        Projection : temporary(Projection), $ ; grid attributes
                        GridOrigin : temporary(GridOrigin), $
                        GridSpacing : temporary(GridSpacing), $
                        GridSpacingUnit : temporary(GridSpacingUnit), $
                        GridSpan : temporary(GridSpan), $
                        GridSpanUnit : temporary(GridSpanUnit), $
                        DeltaFullLatitude : temporary(DeltaFullLatitude), $
                        DeltaFullLongitude : temporary(DeltaFullLongitude), $
                        L3Algorithm : temporary(L3Algorithm) $
                        }



        end                     ; 8-day type
    endcase ; NadirDayGrid file type case
endif ; end if NadirDayGrid found

grid = all_grids[2] ; NadirNightGrid

; this section repeats for all grid types except the
; names of output structures change to match the grid name
nspos = where(names eq grid)
if nspos ne -1 then begin
    case type of
        'daily' : begin         ; separate by type
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'SurfacePressure')
            SurfacePressure = h5d_read(dataid)
            get_data_attributes,dataid,size(SurfacePressure,/type),attrib7
            h5d_close,dataid

            if species_name eq 'TATM' then begin
                dataid = h5d_open(fieldid,'SurfaceTemperature')
                SurfaceTemperature = h5d_read(dataid)
                get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib8
                h5d_close,dataid
            endif else begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid
            endelse

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'TriangleVertDistMax')
            TriangleVertDistMax = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GlobalSurveyL3Algorithm')
            GlobalSurveyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid
            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    nadirnightgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                SurfaceTemperature : temporary(SurfaceTemperature), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }


                end
                'O3' : begin
                    nadirnightgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }
                end
                else : begin
                    nadirnightgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }

                end
            endcase             ; daily species case for output
        end                     ; end daily file case for NadirGrid
        'monthly' : begin       ; monthly file case for NadirGrid
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            if species_name ne 'TATM' then begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensDataCount')
                TotColDensDataCount = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensStdDeviation')
                TotColDensStdDeviation = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMaximum')
                TotColDensMaximum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMinimum')
                TotColDensMinimum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
                h5d_close,dataid
            endif

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'MonthlyL3Algorithm')
            MonthlyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    nadirnightgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }


                end
                'O3' : begin
                    nadirnightgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
                else : begin
                    nadirnightgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
            endcase             ; montly species case for output

        end
        '8d' : begin
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotalColumnDensity')
            TotalColumnDensity = h5d_read(dataid)
            get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensDataCount')
            TotColDensDataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensStdDeviation')
            TotColDensStdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMaximum')
            TotColDensMaximum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMinimum')
            TotColDensMinimum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
            h5d_close,dataid

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'8-DayL3Algorithm')
            L3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            nadirnightgrid = { $
                        species : temporary(species), $
                        AtSurface : temporary(AtSurface), $
                        DataCount : temporary(DataCount), $
                        StdDeviation : temporary(StdDeviation), $
                        Maximum : temporary(Maximum), $
                        Minimum : temporary(Minimum), $
                        Latitude : temporary(Latitude), $
                        Longitude : temporary(Longitude), $
                        Pressure : temporary(Pressure), $
                        TotalColumnDensity : temporary(TotalColumnDensity), $
                        TotColDensDataCount : temporary(TotColDensDataCount), $
                        TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                        TotColDensMaximum : temporary(TotColDensMaximum), $
                        TotColDensMinimum : temporary(TotColDensMinimum), $
                        Projection : temporary(Projection), $ ; grid attributes
                        GridOrigin : temporary(GridOrigin), $
                        GridSpacing : temporary(GridSpacing), $
                        GridSpacingUnit : temporary(GridSpacingUnit), $
                        GridSpan : temporary(GridSpan), $
                        GridSpanUnit : temporary(GridSpanUnit), $
                        DeltaFullLatitude : temporary(DeltaFullLatitude), $
                        DeltaFullLongitude : temporary(DeltaFullLongitude), $
                        L3Algorithm : temporary(L3Algorithm) $
                        }



        end                     ; 8-day type
    endcase ; NadirNightGrid file type case
endif ; end if NadirNightGrid found

grid = all_grids[3] ; LimbGrid

; this section repeats for all grid types except the
; names of output structures change to match the grid name
nspos = where(names eq grid)
if nspos ne -1 then begin
    case type of
        'daily' : begin         ; separate by type
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'SurfacePressure')
            SurfacePressure = h5d_read(dataid)
            get_data_attributes,dataid,size(SurfacePressure,/type),attrib7
            h5d_close,dataid

            if species_name eq 'TATM' then begin
                dataid = h5d_open(fieldid,'SurfaceTemperature')
                SurfaceTemperature = h5d_read(dataid)
                get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib8
                h5d_close,dataid
            endif else begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid
            endelse

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'TriangleVertDistMax')
            TriangleVertDistMax = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GlobalSurveyL3Algorithm')
            GlobalSurveyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid
            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    limbgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                SurfaceTemperature : temporary(SurfaceTemperature), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }


                end
                'O3' : begin
                    limbgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }
                end
                else : begin
                    limbgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }

                end
            endcase             ; daily species case for output
        end                     ; end daily file case for NadirGrid
        'monthly' : begin       ; monthly file case for NadirGrid
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            if species_name ne 'TATM' then begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensDataCount')
                TotColDensDataCount = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensStdDeviation')
                TotColDensStdDeviation = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMaximum')
                TotColDensMaximum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMinimum')
                TotColDensMinimum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
                h5d_close,dataid
            endif

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'MonthlyL3Algorithm')
            MonthlyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    limbgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }


                end
                'O3' : begin
                    limbgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
                else : begin
                    limbgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
            endcase             ; montly species case for output

        end
        '8d' : begin
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotalColumnDensity')
            TotalColumnDensity = h5d_read(dataid)
            get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensDataCount')
            TotColDensDataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensStdDeviation')
            TotColDensStdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMaximum')
            TotColDensMaximum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMinimum')
            TotColDensMinimum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
            h5d_close,dataid

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'8-DayL3Algorithm')
            L3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            limbgrid = { $
                        species : temporary(species), $
                        AtSurface : temporary(AtSurface), $
                        DataCount : temporary(DataCount), $
                        StdDeviation : temporary(StdDeviation), $
                        Maximum : temporary(Maximum), $
                        Minimum : temporary(Minimum), $
                        Latitude : temporary(Latitude), $
                        Longitude : temporary(Longitude), $
                        Pressure : temporary(Pressure), $
                        TotalColumnDensity : temporary(TotalColumnDensity), $
                        TotColDensDataCount : temporary(TotColDensDataCount), $
                        TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                        TotColDensMaximum : temporary(TotColDensMaximum), $
                        TotColDensMinimum : temporary(TotColDensMinimum), $
                        Projection : temporary(Projection), $ ; grid attributes
                        GridOrigin : temporary(GridOrigin), $
                        GridSpacing : temporary(GridSpacing), $
                        GridSpacingUnit : temporary(GridSpacingUnit), $
                        GridSpan : temporary(GridSpan), $
                        GridSpanUnit : temporary(GridSpanUnit), $
                        DeltaFullLatitude : temporary(DeltaFullLatitude), $
                        DeltaFullLongitude : temporary(DeltaFullLongitude), $
                        L3Algorithm : temporary(L3Algorithm) $
                        }



        end                     ; 8-day type
    endcase ; LimbGrid file type case
endif ; end if LiimbGrid found

grid = all_grids[4] ; LimbDayGrid

; this section repeats for all grid types except the
; names of output structures change to match the grid name
nspos = where(names eq grid)
if nspos ne -1 then begin
    case type of
        'daily' : begin         ; separate by type
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'SurfacePressure')
            SurfacePressure = h5d_read(dataid)
            get_data_attributes,dataid,size(SurfacePressure,/type),attrib7
            h5d_close,dataid

            if species_name eq 'TATM' then begin
                dataid = h5d_open(fieldid,'SurfaceTemperature')
                SurfaceTemperature = h5d_read(dataid)
                get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib8
                h5d_close,dataid
            endif else begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid
            endelse

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'TriangleVertDistMax')
            TriangleVertDistMax = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GlobalSurveyL3Algorithm')
            GlobalSurveyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid
            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    limbdaygrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                SurfaceTemperature : temporary(SurfaceTemperature), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }


                end
                'O3' : begin
                    limbdaygrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }
                end
                else : begin
                    limbdaygrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }

                end
            endcase             ; daily species case for output
        end                     ; end daily file case for NadirGrid
        'monthly' : begin       ; monthly file case for NadirGrid
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            if species_name ne 'TATM' then begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensDataCount')
                TotColDensDataCount = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensStdDeviation')
                TotColDensStdDeviation = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMaximum')
                TotColDensMaximum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMinimum')
                TotColDensMinimum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
                h5d_close,dataid
            endif

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'MonthlyL3Algorithm')
            MonthlyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    limbdaygrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }


                end
                'O3' : begin
                    limbdaygrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
                else : begin
                    limbdaygrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
            endcase             ; montly species case for output

        end
        '8d' : begin
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotalColumnDensity')
            TotalColumnDensity = h5d_read(dataid)
            get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensDataCount')
            TotColDensDataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensStdDeviation')
            TotColDensStdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMaximum')
            TotColDensMaximum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMinimum')
            TotColDensMinimum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
            h5d_close,dataid

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'8-DayL3Algorithm')
            L3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            limbdaygrid = { $
                        species : temporary(species), $
                        AtSurface : temporary(AtSurface), $
                        DataCount : temporary(DataCount), $
                        StdDeviation : temporary(StdDeviation), $
                        Maximum : temporary(Maximum), $
                        Minimum : temporary(Minimum), $
                        Latitude : temporary(Latitude), $
                        Longitude : temporary(Longitude), $
                        Pressure : temporary(Pressure), $
                        TotalColumnDensity : temporary(TotalColumnDensity), $
                        TotColDensDataCount : temporary(TotColDensDataCount), $
                        TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                        TotColDensMaximum : temporary(TotColDensMaximum), $
                        TotColDensMinimum : temporary(TotColDensMinimum), $
                        Projection : temporary(Projection), $ ; grid attributes
                        GridOrigin : temporary(GridOrigin), $
                        GridSpacing : temporary(GridSpacing), $
                        GridSpacingUnit : temporary(GridSpacingUnit), $
                        GridSpan : temporary(GridSpan), $
                        GridSpanUnit : temporary(GridSpanUnit), $
                        DeltaFullLatitude : temporary(DeltaFullLatitude), $
                        DeltaFullLongitude : temporary(DeltaFullLongitude), $
                        L3Algorithm : temporary(L3Algorithm) $
                        }



        end                     ; 8-day type
    endcase ; LimbDayGrid file type case
endif ; end if LimbDayGrid found

grid = all_grids[5] ; LimbNightGrid

; this section repeats for all grid types except the
; names of output structures change to match the grid name
nspos = where(names eq grid)
if nspos ne -1 then begin
    case type of
        'daily' : begin         ; separate by type
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'SurfacePressure')
            SurfacePressure = h5d_read(dataid)
            get_data_attributes,dataid,size(SurfacePressure,/type),attrib7
            h5d_close,dataid

            if species_name eq 'TATM' then begin
                dataid = h5d_open(fieldid,'SurfaceTemperature')
                SurfaceTemperature = h5d_read(dataid)
                get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib8
                h5d_close,dataid
            endif else begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid
            endelse

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'TriangleVertDistMax')
            TriangleVertDistMax = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GlobalSurveyL3Algorithm')
            GlobalSurveyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid
            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    limbnightgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                SurfaceTemperature : temporary(SurfaceTemperature), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }


                end
                'O3' : begin
                    limbnightgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }
                end
                else : begin
                    limbnightgrid = { $
                                species : temporary(species), $
                                AtSurface : temporary(AtSurface), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                SurfacePressure : temporary(SurfacePressure), $
                                TotalColumnDensitye : temporary(TotalColumnDensity), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                GlobalSurveyL3Algorithm : temporary(GlobalSurveyL3Algorithm), $
                                TriangleVertDistMax : temporary(TriangleVertDistMax) $
                                }

                end
            endcase             ; daily species case for output
        end                     ; end daily file case for NadirGrid
        'monthly' : begin       ; monthly file case for NadirGrid
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            if species_name eq 'O3' then begin
                dataid = h5d_open(fieldid,'OzoneTropColumn')
                OzoneTropColumn = h5d_read(dataid)
                get_data_attributes,dataid,size(OzoneTropColumn,/type),attrib5
                h5d_close,dataid
            endif

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            if species_name ne 'TATM' then begin
                dataid = h5d_open(fieldid,'TotalColumnDensity')
                TotalColumnDensity = h5d_read(dataid)
                get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensDataCount')
                TotColDensDataCount = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensStdDeviation')
                TotColDensStdDeviation = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMaximum')
                TotColDensMaximum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
                h5d_close,dataid

                dataid = h5d_open(fieldid,'TotColDensMinimum')
                TotColDensMinimum = h5d_read(dataid)
                get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
                h5d_close,dataid
            endif

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'MonthlyL3Algorithm')
            MonthlyL3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            case species_name of     ; special cases for TATM and O3
                'TATM' : begin
                    limbnightgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }


                end
                'O3' : begin
                    limbnightgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                OzoneTropColumn : temporary(OzoneTropColumn), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
                else : begin
                    limbnightgrid = { $
                                species : temporary(species), $
                                DataCount : temporary(DataCount), $
                                StdDeviation : temporary(StdDeviation), $
                                Maximum : temporary(Maximum), $
                                Minimum : temporary(Minimum), $
                                Latitude : temporary(Latitude), $
                                Longitude : temporary(Longitude), $
                                Pressure : temporary(Pressure), $
                                TotalColumnDensity : temporary(TotalColumnDensity), $
                                TotColDensDataCount : temporary(TotColDensDataCount), $
                                TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                                TotColDensMaximum : temporary(TotColDensMaximum), $
                                TotColDensMinimum : temporary(TotColDensMinimum), $
                                Projection : temporary(Projection), $ ; grid attributes
                                GridOrigin : temporary(GridOrigin), $
                                GridSpacing : temporary(GridSpacing), $
                                GridSpacingUnit : temporary(GridSpacingUnit), $
                                GridSpan : temporary(GridSpan), $
                                GridSpanUnit : temporary(GridSpanUnit), $
                                DeltaFullLatitude : temporary(DeltaFullLatitude), $
                                DeltaFullLongitude : temporary(DeltaFullLongitude), $
                                MonthlyL3Algorithm : temporary(MonthlyL3Algorithm) $
                                }

                end
            endcase             ; montly species case for output

        end
        '8d' : begin
            specid = h5g_open(gridid,grid)
            fieldid = h5g_open(specid,'Data Fields')

            dataid = h5d_open(fieldid,species_name)
            species = h5d_read(dataid)
            get_data_attributes,dataid,size(species,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'AtSurface')
            AtSurface = h5d_read(dataid)
            get_data_attributes,dataid,size(AtSurface,/type),attrib2
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'DataCount')
            DataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(DataCount,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'StdDeviation')
            StdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(StdDeviation,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Maximum')
            Maximum = h5d_read(dataid)
            get_data_attributes,dataid,size(Maximum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,species_name+'Minimum')
            Minimum = h5d_read(dataid)
            get_data_attributes,dataid,size(Minimum,/type),attrib1
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Latitude')
            Latitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Latitude,/type),attrib3
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Longitude')
            Longitude = h5d_read(dataid)
            get_data_attributes,dataid,size(Longitude,/type),attrib4
            h5d_close,dataid

            dataid = h5d_open(fieldid,'Pressure')
            Pressure = h5d_read(dataid)
            get_data_attributes,dataid,size(Pressure,/type),attrib6
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotalColumnDensity')
            TotalColumnDensity = h5d_read(dataid)
            get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensDataCount')
            TotColDensDataCount = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensDataCount,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensStdDeviation')
            TotColDensStdDeviation = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensStdDeviation,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMaximum')
            TotColDensMaximum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMaximum,/type),attrib8
            h5d_close,dataid

            dataid = h5d_open(fieldid,'TotColDensMinimum')
            TotColDensMinimum = h5d_read(dataid)
            get_data_attributes,dataid,size(TotColDensMinimum,/type),attrib8
            h5d_close,dataid

            h5g_close,fieldid
;
; get grid level attributes
;
            id = h5a_open_name(specid,'Projection')
            projection = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridOrigin')
            GridOrigin = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacing')
            GridSpacing = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpacingUnit')
            GridSpacingUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpan')
            GridSpan = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'GridSpanUnit')
            GridSpanUnit = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLatitude')
            DeltaFullLatitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'DeltaFullLongitude')
            DeltaFullLongitude = h5a_read(id)
            h5a_close,id

            id = h5a_open_name(specid,'8-DayL3Algorithm')
            L3Algorithm = h5a_read(id)
            h5a_close,id

            h5g_close,specid

            ; make output structure
            limbnightgrid = { $
                        species : temporary(species), $
                        AtSurface : temporary(AtSurface), $
                        DataCount : temporary(DataCount), $
                        StdDeviation : temporary(StdDeviation), $
                        Maximum : temporary(Maximum), $
                        Minimum : temporary(Minimum), $
                        Latitude : temporary(Latitude), $
                        Longitude : temporary(Longitude), $
                        Pressure : temporary(Pressure), $
                        TotalColumnDensity : temporary(TotalColumnDensity), $
                        TotColDensDataCount : temporary(TotColDensDataCount), $
                        TotColDensStdDeviation : temporary(TotColDensStdDeviation), $
                        TotColDensMaximum : temporary(TotColDensMaximum), $
                        TotColDensMinimum : temporary(TotColDensMinimum), $
                        Projection : temporary(Projection), $ ; grid attributes
                        GridOrigin : temporary(GridOrigin), $
                        GridSpacing : temporary(GridSpacing), $
                        GridSpacingUnit : temporary(GridSpacingUnit), $
                        GridSpan : temporary(GridSpan), $
                        GridSpanUnit : temporary(GridSpanUnit), $
                        DeltaFullLatitude : temporary(DeltaFullLatitude), $
                        DeltaFullLongitude : temporary(DeltaFullLongitude), $
                        L3Algorithm : temporary(L3Algorithm) $
                        }



        end                     ; 8-day type
    endcase ; LimbNightGrid file type case
endif ; end if LimbNightGrid found


h5g_close,gridid



;
; get 'Additional' information
;
aid = h5g_open(hdfid,'ADDITIONAL')
faaid = h5g_open(aid,'FILE_ATTRIBUTES')

if read_ph ne 0 then begin
    catch,error_status
    IF Error_status NE 0 THEN BEGIN
      ;PRINT, 'Error index: ', Error_status
      ;PRINT, 'Error message: ', !ERROR_STATE.MSG
      ; Handle the error by extending A:
      PRINT, ' Production History not available in this file'
      ProductionHistory = ''
      CATCH, /CANCEL
      goto, next
   ENDIF
    dataid = h5d_open(faaid,'ProductionHistory')
    ProductionHistory = h5d_read(dataid)
    h5d_close,dataid
endif else begin
    ProductionHistory = ''
endelse

next:

id = h5a_open_name(faaid,'InstrumentName')
InstrumentName = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'ProcessLevel')
ProcessLevel = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'OrbitNumber')
OrbitNumber = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'OrbitPeriod')
OrbitPeriod = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'TAI93At0zOfGranule')
TAI93At0zOfGranule = h5a_read(id)
h5a_close,id


id = h5a_open_name(faaid,'GlobalSurveyNumber')
GlobalSurveyNumber = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleMonth')
GranuleMonth = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleDay')
GranuleDay = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleYear')
GranuleYear = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'SurveyMode')
SurveyMode = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'PGEVersion')
PGEVersion = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'StartUTC')
StartUTC = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'EndUTC')
EndUTC = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'Period')
Period = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'Command_Seq_ID')
Command_Seq_ID = h5a_read(id)
h5a_close,id

; clean up
h5g_close,faaid
h5g_close,aid
h5g_close,hdfid

file_attributes = { $
                  ProductionHistory : temporary(ProductionHistory), $
                  InstrumentName : temporary(InstrumentName), $
                  ProcessLevel : temporary(ProcessLevel), $
                  OrbitNumber : temporary(OrbitNumber), $
                  OrbitPeriod : temporary(OrbitPeriod), $
                  TAI93At0zOfGranule : temporary(TAI93At0zOfGranule), $
                  GlobalSurveyNumber : temporary(GlobalSurveyNumber), $
                  GranuleMonth : temporary(GranuleMonth), $
                  GranuleDay : temporary(GranuleDay), $
                  GranuleYear : temporary(GranuleYear), $
                  SurveyMode : temporary(SurveyMode), $
                  PGEVersion : temporary(PGEVersion), $
                  StartUTC : temporary(StartUTC), $
                  EndUTC : temporary(EndUTC), $
                  Period : temporary(Period), $
                  Command_Seq_ID : temporary(Command_Seq_ID)    }

; last bit of file
hdfinfid = h5g_open(fid,'HDFEOS INFORMATION')

dataid = h5d_open(hdfinfid,'StructMetadata.0')
StructMetadata = h5d_read(dataid)
h5d_close,dataid

dataid = h5d_open(hdfinfid,'coremetadata')
coremetadata = h5d_read(dataid)
h5d_close,dataid

id = h5a_open_name(hdfinfid,'HDFEOSVersion')
HDFEOSVersion = h5a_read(id)
h5a_close,id

h5g_close,hdfinfid
h5f_close,fid

hdfeos_data = { $
              StructMetadata : temporary(StructMetadata), $
              coremetadata : temporary(coremetadata), $
              HDFEOSVersion : temporary(HDFEOSVersion) }

end
